package Workflow::App;

use strict;
use base qw( MT::App::CMS );

use MT::Util qw( format_ts perl_sha1_digest_hex );
use CustomFields::Util qw( get_meta );
use Workflow::Util qw( can_workflow get_lastest_revision get_lastest_revision_object );

sub init_request {
    my $app = shift;
    $app->SUPER::init_request(@_);
    $app->add_methods( preview => \&preview );

    my $plugin = MT->component('Workflow');
    my $workflow_preview_mode = $plugin->get_config_value('workflow_preview_mode', 'blog:'.$app->blog->id);
    $app->add_methods( $workflow_preview_mode => \&preview_without_login ) if $workflow_preview_mode;

    $app->{ default_mode } = 'main';
    $app->{ requires_login } = 0;
}

sub send_mail {
    my $app = shift;
    return $app->load_tmpl( '../plugins/Workflow/tmpl/send_mail.tmpl', $app->param );
}

sub preview {
    my $app = shift;

    if ($app->param('__mode') eq 'workflow_preview') {
        my $reviewer = $app->can_do('approve_entry');
        return $app->errtrans('Permission denied.') unless $reviewer;
    } else {
        return $app->errtrans('Permission denied.');
    }
    return _build_entry_preview($app);
}

sub preview_without_login {
    my $app = shift;

    my $plugin = MT->component('Workflow');
    my $workflow_preview_mode = $plugin->get_config_value('workflow_preview_mode', 'blog:'.$app->blog->id);
    return $app->errtrans('Permission denied.') if $app->param('__mode') ne $workflow_preview_mode;

    return _build_entry_preview($app);
}

sub _build_entry_preview {
    my $app = shift;

    my $q           = $app->param;
    my $class       = $q->param('_type');
    my $type        = $q->param('_type');
#    my $type        = 'entry'; # $q->param('_type') || 'entry';
    my $entry_class = $app->model($type);
    my $blog_id     = $q->param('blog_id');
    my $blog        = $app->blog;
    my $id          = $q->param('id');

    my ($entry, %param);
    my $user_id;
    my $cat;

    my $is_revision = 0;
    my $obj;
    my $model = $app->model($type);

    my $revision = get_lastest_revision($app);
    return $app->error( "No Revision." ) if !$revision;

    my @objs = $model->load($id);
    $obj = shift @objs || undef;
    if ($obj) {
        my $rev = $obj->load_revision( { rev_number => $revision->rev_number } );
        if ( $rev && @$rev ) {
           $entry = {};
           $entry = $rev->[0];
           $is_revision = 1;
        }
    }

    my $primary;
    my $secondary;
    my @category_list;
    if ($is_revision) {
        $primary = $obj->cache_property( 'category' );
        $secondary = $obj->cache_property( 'categories' );
    } else {
        $primary = _get_primary_categories($id);
        $secondary = _get_secondary_categories($id);
    }
    push @category_list, $primary->id if $primary;
    if ($secondary) {
        foreach my $c (@{$secondary}) {
            push @category_list, $c->id;
        }
    }

    my $cat_id;
    my %places;

    my $cats = join(',',@category_list);
    if ( defined $cats ) {
        if ( my @cats = grep { $_ =~ /^\d+/ } split /,/, $cats ) {
            $cat_id = $cats[0];
            %places = map { $_ => 1 } @cats;
        }
    }
    my $data = $app->SUPER::_build_category_list(
        blog_id => $blog_id,
        markers => 1,
        type    => $model->container_type,
    );
    my $top_cat = $cat_id;
    my @sel_cats;
    my $cat_tree = [];

    if ( $class eq 'page' ) {
        push @$cat_tree,
            {
            id       => -1,
            label    => '/',
            basename => '/',
            path     => [],
            };
        $top_cat ||= -1;
    }
    foreach (@$data) {
        next unless exists $_->{category_id};

    if ( $class eq 'page' ) {
            $_->{category_path_ids} ||= [];
            unshift @{ $_->{category_path_ids} }, -1;
        }
        push @$cat_tree,
            {
            id       => $_->{category_id},
            label    => $_->{category_label} . ( $class eq 'page' ? '/' : '' ),
            basename => $_->{category_basename} . ( $class eq 'page' ? '/' : '' ),
            path   => $_->{category_path_ids} || [],
            fields => $_->{category_fields}   || [],
            };
        push @sel_cats, $_->{category_id}
            if $places{ $_->{category_id} }
                && $_->{category_id} != $cat_id;
    }
    $param{category_tree} = $cat_tree;
    unshift @sel_cats, $top_cat if defined $top_cat && $top_cat ne "";
    $param{selected_category_loop}   = \@sel_cats;
    $param{have_multiple_categories} = scalar @$data > 1;

#    my $tag_delim = chr( $app->user->entry_prefs->{tag_delim} );
    my $tag_delim = ',';

    my @tag_names = $entry->tags;
    if (@tag_names) {
        my @tags;
        foreach my $tag_name (@tag_names) {

            next if $tag_name =~ m/^@/;
            my $tag = MT::Tag->new;
            $tag->name($tag_name);
            push @tags, $tag;
        }
        $entry->{__tags} = \@tag_names;
        $entry->{__tag_objects} = \@tags;
    }

    my $date = format_ts( "%Y-%m-%d", $entry->authored_on, $blog, undef );
    my $time = format_ts( "%H:%M:%S", $entry->authored_on, $blog, undef );
    my $ts = $date . $time;
    $ts =~ s/\D//g;

    my $preview_basename = preview_object_basename($app);

    require MT::TemplateMap;
    require MT::Template;
    my $at = $class eq 'page' ? 'Page' : 'Individual';
    my $tmpl_map = MT::TemplateMap->load(
        {
            archive_type => $at,
            is_preferred => 1,
            blog_id => $blog_id,
        }
    );

    my $tmpl;
    my $fullscreen;
    my $archive_file;
    my $orig_file;
    my $file_ext;
    if ($tmpl_map) {
        $tmpl         = MT::Template->load( $tmpl_map->template_id );
        $file_ext = $blog->file_extension || '';
        $archive_file = $entry->archive_file;

        my $blog_path = $class eq 'page' ?
            $blog->site_path :
            ($blog->archive_path || $blog->site_path);
        $archive_file = File::Spec->catfile( $blog_path, $archive_file );
        require File::Basename;
        my $path;
        ( $orig_file, $path ) = File::Basename::fileparse( $archive_file );
        $file_ext = '.' . $file_ext if $file_ext ne '';
        $archive_file = File::Spec->catfile( $path, $preview_basename . $file_ext );
    }
    else {
        $tmpl       = $app->load_tmpl('preview_entry_content.tmpl');
        $fullscreen = 1;
    }

    return $app->error($app->translate('Can\'t load template.')) unless $tmpl;

    MT::Util::translate_naughty_words($entry);

    my @data = ( { data_name => 'author_id', data_value => $user_id } );

    _set_customfield_objs($app, $entry, \@data);

    my $ctx = $tmpl->context;
    $ctx->stash( 'entry', $entry );
    $ctx->stash( 'blog',  $blog );
    $ctx->stash( 'category', $cat ) if $cat;
    $ctx->{current_timestamp} = $ts;
    $ctx->{current_archive_type} = $at;
    $ctx->var('entry_template',    1);
    $ctx->var('archive_template',  1);
    $ctx->var('entry_template',    1);
    $ctx->var('feedback_template', 1);
    $ctx->var('archive_class',     'entry-archive');
    $ctx->var('preview_template',  1);
    my $html = $tmpl->output;
    unless ( defined($html) ) {
        my $preview_error = $app->translate( "Publish error: [_1]",
            MT::Util::encode_html( $tmpl->errstr ) );
        $param{preview_error} = $preview_error;
        my $tmpl_plain = $app->load_tmpl('preview_entry_content.tmpl');
        $tmpl->text( $tmpl_plain->text );
        $html = $tmpl->output;
        defined($html)
          or return $app->error(
            $app->translate( "Publish error: [_1]", $tmpl->errstr ) );
        $fullscreen = 1;
    }

    # If MT is configured to do 'local' previews, convert all
    # the normal blog URLs into the domain used by MT itself (ie,
    # blog is published to www.example.com, which is a different
    # server from where MT runs, mt.example.com; previews therefore
    # should occur locally, so replace all http://www.example.com/
    # with http://mt.example.com/).
    my ($old_url, $new_url);
    if ($app->config('LocalPreviews')) {
        $old_url = $blog->site_url;
        $old_url =~ s!^(https?://[^/]+?/)(.*)?!$1!;
        $new_url = $app->base . '/';
        $html =~ s!\Q$old_url\E!$new_url!g;
    }

    if ( !$fullscreen ) {
        my $fmgr = $blog->file_mgr;

        ## Determine if we need to build directory structure,
        ## and build it if we do. DirUmask determines
        ## directory permissions.
        require File::Basename;
        my $path = File::Basename::dirname($archive_file);
        $path =~ s!/$!!
          unless $path eq '/';    ## OS X doesn't like / at the end in mkdir().
        unless ( $fmgr->exists($path) ) {
            $fmgr->mkpath($path);
        }

        if ( $fmgr->exists($path) && $fmgr->can_write($path) ) {
            $fmgr->put_data( $html, $archive_file );
            $param{preview_file} = $preview_basename;
            my $preview_url = $entry->archive_url;
            $preview_url =~ s! / \Q$orig_file\E ( /? ) $!/$preview_basename$file_ext$1!x;

            # We also have to translate the URL used for the
            # published file to be on the MT app domain.
            if (defined $new_url) {
                $preview_url =~ s!^\Q$old_url\E!$new_url!;
            }

            $preview_url =~ s!^https?:(.*)!$1!;

            $param{preview_url}  = $preview_url;

            # we have to make a record of this preview just in case it
            # isn't cleaned up by re-editing, saving or cancelling on
            # by the user.
            require MT::Session;
            my $sess_obj = MT::Session->get_by_key(
                {
                    id   => $preview_basename,
                    kind => 'TF',                # TF = Temporary File
                    name => $archive_file,
                }
            );
            $sess_obj->start(time);
            $sess_obj->save;
        }
        else {
            $fullscreen = 1;
            $param{preview_error} = $app->translate(
                "Unable to create preview file in this location: [_1]", $path );
            my $tmpl_plain = $app->load_tmpl('preview_entry_content.tmpl');
            $tmpl->text( $tmpl_plain->text );
            $tmpl->reset_tokens;
            $html = $tmpl->output;
            $param{preview_body} = $html;
        }
    }
    else {
        $param{preview_body} = $html;
    }

    $param{id} = $id if $id;
    $param{new_object} = $param{id} ? 0 : 1;
    $param{title} = $entry->title;
    my $cols = $entry_class->column_names;

    for my $col (@$cols) {
        next
          if $col eq 'created_on'
          || $col eq 'created_by'
          || $col eq 'modified_on'
          || $col eq 'modified_by'
          || $col eq 'authored_on'
          || $col eq 'author_id'
          || $col eq 'pinged_urls'
          || $col eq 'tangent_cache'
          || $col eq 'template_id'
          || $col eq 'class'
          || $col eq 'meta'
          || $col eq 'comment_count'
          || $col eq 'ping_count'
          || $col eq 'current_revision';
        push @data,
          {
            data_name  => $col,
            data_value => scalar $entry->$col
          };
    }

    my $data2;
    my @obj_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $entry->id });
    $data2->{include_asset_ids} = join(',', @obj_assets) || '';
    $data2->{tags} = join(',', @tag_names);

    # プライマリーカテゴリを登録
    my $primary_cat_id = shift @category_list;
    my @preview_category_list;
    push @preview_category_list, $primary_cat_id;

    # 他のカテゴリを登録
    for my $cat_id ( @category_list ) {
        push @preview_category_list, $cat_id if $primary_cat_id != $cat_id;
    }
    $data2->{category_ids} = join(',', @preview_category_list ) if @preview_category_list;

    $data2->{authored_on_date} = format_ts( "%Y-%m-%d", $entry->authored_on, $blog, undef );
    $data2->{authored_on_time} = format_ts( "%H:%M:%S", $entry->authored_on, $blog, undef );

    $data2->{save_revision} = 1;
    $data2->{'revision-note'} = '';
    $data2->{basename_manual} = '';
    $data2->{basename_old} = '';

    for my $data (
        qw( authored_on_date authored_on_time basename_manual basename_old category_ids tags include_asset_ids save_revision revision-note )
      )
    {
        push @data,
          {
            data_name  => $data,
            data_value => scalar $data2->{$data}
          };
    }

    $param{entry_loop} = \@data;
    my $list_mode;
    my $list_title;
    if ( $class eq 'page' ) {
        $list_title = 'Pages';
        $list_mode  = 'list_page';
    } else {
        $list_title = 'Entries';
        $list_mode  = 'list_entry';
    }
    if ($id) {
        $app->add_breadcrumb(
            $app->translate($list_title),
            $app->uri(
                'mode' => $list_mode,
                args   => { blog_id => $blog_id }
            )
        );
        $app->add_breadcrumb( $entry->title || $app->translate('(untitled)') );
    } else {
        $app->add_breadcrumb( $app->translate($list_title),
            $app->uri( 'mode' => $list_mode, args => { blog_id => $blog_id } )
        );
        $app->add_breadcrumb(
            $app->translate( 'New [_1]', $entry_class->class_label ) );
        $param{nav_new_entry} = 1;
    }

    $param{object_type}  = $class;
    $param{object_label} = $entry_class->class_label;

    $param{diff_view} = $app->param('rev_numbers') || $app->param('collision');
    $param{collision} = 1;
    if(my @rev_numbers = split /,/, $app->param('rev_numbers')) {
        $param{comparing_revisions} = 1;
        $param{rev_a} = $rev_numbers[0];
        $param{rev_b} = $rev_numbers[1];
    }
    $param{dirty} = $q->param('dirty') ? 1 : 0;

    if ($entry->status == 3) {
        $param{status_review} = 1;
    } elsif ($entry->status == 1) {
        $param{status_draft} = 1;
    } elsif ($entry->status == 20) {
        $param{status_publish_review} = 1;
    } elsif ($entry->status == 10) {
        $param{status_publish_draft} = 1;
    }

    if ($fullscreen) {
        return $app->load_tmpl( 'preview_entry.tmpl', \%param );
    }
    else {
        $app->request('preview_object', $entry);

        if($q->param('__mode') eq 'workflow_preview') {
            return $app->load_tmpl( '../plugins/Workflow/tmpl/workflow_preview.tmpl', \%param );
        } else {
            return $app->load_tmpl( '../plugins/Workflow/tmpl/preview.tmpl', \%param );
        }
    }
}

sub _get_primary_categories {
    my $id = shift;

    my $place_primary = MT::Placement->load(
        {
            entry_id => $id,
            is_primary => 1
        }
    );
    return $place_primary;
}

sub _get_secondary_categories {
    my $id = shift;

    my @place_secondary = MT::Placement->load(
        {
            entry_id   => $id,
            is_primary => 0
        }
    );
    return @place_secondary;
}

sub _set_customfield_objs {

    my ($app, $entry, $data) = @_;
    my $q = $app->param;

    if ($entry->has_meta) {
        my $meta = get_meta($entry);
        foreach my $basename (keys %$meta) {
            my $field = MT->model('field')->load(
                { basename => $basename, blog_id => [ $entry->blog_id, 0 ] }
            );
            next unless $entry->has_meta('field.'.$basename);

            my $value = $entry->meta('field.'.$basename);

            my $type = '';
            if ($field->type eq 'datetime') {
                $value =~ s/^\s*$//;
                my $date = '';
                my $time = '';
                if ($value =~ m/^(\d{4}-\d{2}-\d{2}) (\d{2}:\d{2}:\d{2})$/) {
                    $date = $1;
                    $time = $2;
                }
                push @$data,
                  {
                      data_name  => "d_customfield_$basename",
                      data_value => scalar $date
                  };
                push @$data,
                  {
                      data_name  => "t_customfield_$basename",
                      data_value => scalar $time
                  };
                next;
            }
            push @$data,
              {
                  data_name  => "${type}customfield_$basename",
                  data_value => scalar $value
              };
        }
        push @$data, { data_name => 'customfield_beacon', data_value => 1 };
    }

    1;
}

sub preview_object_basename {
    my $app = shift;
    my $q   = $app->param;
    my @parts;
    my $blog    = $app->blog;
    my $blog_id = $blog->id if $blog;
    my $id      = $q->param('id');
    push @parts, $blog_id || 0;
    push @parts, $id || 0;
    push @parts, $q->param('_type');
    push @parts, $app->config->SecretToken;
    my $data = join ",", @parts;
    return 'mt-preview-' . perl_sha1_digest_hex($data);
}

1;
